package sun.awt.datatransfer;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.io.Serializable;
import java.util.Comparator;
import java.nio.charset.CharsetEncoder;
import java.io.BufferedReader;
import java.util.TreeSet;
import java.util.SortedSet;
import java.util.LinkedHashSet;
import java.awt.EventQueue;
import sun.awt.AppContext;
import sun.awt.SunToolkit;
import java.util.Stack;
import java.io.SequenceInputStream;
import javax.imageio.stream.ImageOutputStream;
import javax.imageio.ImageWriter;
import javax.imageio.ImageTypeSpecifier;
import java.awt.Graphics;
import sun.awt.image.ImageRepresentation;
import java.awt.image.ColorModel;
import sun.awt.image.ToolkitImage;
import java.awt.image.RenderedImage;
import java.awt.image.BufferedImage;
import javax.imageio.ImageReadParam;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.ImageReader;
import javax.imageio.ImageIO;
import java.lang.reflect.Modifier;
import java.security.PrivilegedAction;
import java.lang.reflect.Constructor;
import java.io.StringReader;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ByteArrayInputStream;
import java.io.FilePermission;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.ProtectionDomain;
import java.security.PrivilegedExceptionAction;
import java.io.ObjectOutputStream;
import java.io.Reader;
import java.nio.CharBuffer;
import java.nio.ByteBuffer;
import java.awt.Image;
import java.net.URISyntaxException;
import java.net.URI;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.SortedMap;
import java.awt.datatransfer.Transferable;
import java.util.Collections;
import java.util.List;
import java.awt.datatransfer.FlavorTable;
import java.awt.datatransfer.FlavorMap;
import java.nio.charset.UnsupportedCharsetException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.Charset;
import java.awt.Toolkit;
import sun.awt.ComponentFactory;
import java.io.File;
import java.util.ArrayList;
import sun.util.logging.PlatformLogger;
import java.util.Set;
import java.util.Map;
import java.awt.datatransfer.DataFlavor;

public abstract class DataTransferer
{
    public static final DataFlavor plainTextStringFlavor;
    public static final DataFlavor javaTextEncodingFlavor;
    private static final Map textMIMESubtypeCharsetSupport;
    private static String defaultEncoding;
    private static final Set textNatives;
    private static final Map nativeCharsets;
    private static final Map nativeEOLNs;
    private static final Map nativeTerminators;
    private static final String DATA_CONVERTER_KEY = "DATA_CONVERTER_KEY";
    private static DataTransferer transferer;
    private static final PlatformLogger dtLog;
    private static final String[] DEPLOYMENT_CACHE_PROPERTIES;
    private static final ArrayList<File> deploymentCacheDirectoryList;
    
    public DataTransferer() {
    }
    
    public static synchronized DataTransferer getInstance() {
        return ((ComponentFactory)Toolkit.getDefaultToolkit()).getDataTransferer();
    }
    
    public static String canonicalName(String charsetName) {
        if (charsetName == null) {
            return null;
        }
        try {
            return Charset.forName(charsetName).name();
        }
        catch (IllegalCharsetNameException ex) {
            return charsetName;
        }
        catch (UnsupportedCharsetException ex2) {
            return charsetName;
        }
    }
    
    public static String getTextCharset(DataFlavor dataFlavor) {
        if (!isFlavorCharsetTextType(dataFlavor)) {
            return null;
        }
        String parameter = dataFlavor.getParameter("charset");
        return (parameter != null) ? parameter : getDefaultTextCharset();
    }
    
    public static String getDefaultTextCharset() {
        if (DataTransferer.defaultEncoding != null) {
            return DataTransferer.defaultEncoding;
        }
        return DataTransferer.defaultEncoding = Charset.defaultCharset().name();
    }
    
    public static boolean doesSubtypeSupportCharset(DataFlavor dataFlavor) {
        if (DataTransferer.dtLog.isLoggable(PlatformLogger.Level.FINE) && !"text".equals(dataFlavor.getPrimaryType())) {
            DataTransferer.dtLog.fine("Assertion (\"text\".equals(flavor.getPrimaryType())) failed");
        }
        String subType = dataFlavor.getSubType();
        if (subType == null) {
            return false;
        }
        Object value = DataTransferer.textMIMESubtypeCharsetSupport.get(subType);
        if (value != null) {
            return value == Boolean.TRUE;
        }
        int n = dataFlavor.getParameter("charset") != null;
        DataTransferer.textMIMESubtypeCharsetSupport.put(subType, n ? Boolean.TRUE : Boolean.FALSE);
        return n;
    }
    
    public static boolean doesSubtypeSupportCharset(String s, String s2) {
        Object value = DataTransferer.textMIMESubtypeCharsetSupport.get(s);
        if (value != null) {
            return value == Boolean.TRUE;
        }
        int n = s2 != null;
        DataTransferer.textMIMESubtypeCharsetSupport.put(s, n ? Boolean.TRUE : Boolean.FALSE);
        return n;
    }
    
    public static boolean isFlavorCharsetTextType(DataFlavor that) {
        if (DataFlavor.stringFlavor.equals(that)) {
            return true;
        }
        if (!"text".equals(that.getPrimaryType()) || !doesSubtypeSupportCharset(that)) {
            return false;
        }
        Class<Object> representationClass = that.getRepresentationClass();
        if (that.isRepresentationClassReader() || String.class.equals(representationClass) || that.isRepresentationClassCharBuffer() || char[].class.equals(representationClass)) {
            return true;
        }
        if (!that.isRepresentationClassInputStream() && !that.isRepresentationClassByteBuffer() && !byte[].class.equals(representationClass)) {
            return false;
        }
        String parameter = that.getParameter("charset");
        return parameter == null || isEncodingSupported(parameter);
    }
    
    public static boolean isFlavorNoncharsetTextType(DataFlavor dataFlavor) {
        return "text".equals(dataFlavor.getPrimaryType()) && !doesSubtypeSupportCharset(dataFlavor) && (dataFlavor.isRepresentationClassInputStream() || dataFlavor.isRepresentationClassByteBuffer() || byte[].class.equals(dataFlavor.getRepresentationClass()));
    }
    
    public static boolean isEncodingSupported(String charsetName) {
        if (charsetName == null) {
            return false;
        }
        try {
            return Charset.isSupported(charsetName);
        }
        catch (IllegalCharsetNameException ex) {
            return false;
        }
    }
    
    public static boolean isRemote(Class<?> clazz) {
        return DataTransferer.RMI.isRemote(clazz);
    }
    
    public static Set<String> standardEncodings() {
        return DataTransferer.StandardEncodingsHolder.standardEncodings;
    }
    
    public static FlavorTable adaptFlavorMap(final FlavorMap flavorMap) {
        if ((flavorMap instanceof FlavorTable)) {
            return (FlavorTable)flavorMap;
        }
        return new FlavorTable() {
            @Override
            public Map getNativesForFlavors(DataFlavor[] array) {
                return flavorMap.getNativesForFlavors(array);
            }
            
            @Override
            public Map getFlavorsForNatives(String[] array) {
                return flavorMap.getFlavorsForNatives(array);
            }
            
            @Override
            public List getNativesForFlavor(DataFlavor dataFlavor) {
                String s = (String)this.getNativesForFlavors(new DataFlavor[] { dataFlavor }).get(dataFlavor);
                if (s != null) {
                    ArrayList list = new ArrayList(1);
                    list.add(s);
                    return list;
                }
                return Collections.EMPTY_LIST;
            }
            
            @Override
            public List getFlavorsForNative(String s) {
                DataFlavor dataFlavor = (DataFlavor)this.getFlavorsForNatives(new String[] { s }).get(s);
                if (dataFlavor != null) {
                    ArrayList list = new ArrayList(1);
                    list.add(dataFlavor);
                    return list;
                }
                return Collections.EMPTY_LIST;
            }
        };
    }
    
    public abstract String getDefaultUnicodeEncoding();
    
    public void registerTextFlavorProperties(String s, String s2, String s3, String s4) {
        Long formatForNativeAsLong = this.getFormatForNativeAsLong(s);
        DataTransferer.textNatives.add(formatForNativeAsLong);
        DataTransferer.nativeCharsets.put(formatForNativeAsLong, (s2 != null && s2.length() != 0) ? s2 : getDefaultTextCharset());
        if (s3 != null && s3.length() != 0 && !s3.equals("\n")) {
            DataTransferer.nativeEOLNs.put(formatForNativeAsLong, s3);
        }
        if (s4 != null && s4.length() != 0) {
            Integer value = Integer.valueOf(s4);
            if ((int)value > 0) {
                DataTransferer.nativeTerminators.put(formatForNativeAsLong, value);
            }
        }
    }
    
    protected boolean isTextFormat(long l) {
        return DataTransferer.textNatives.contains(Long.valueOf(l));
    }
    
    protected String getCharsetForTextFormat(Long n) {
        return (String)DataTransferer.nativeCharsets.get(n);
    }
    
    public abstract boolean isLocaleDependentTextFormat(long p0);
    
    public abstract boolean isFileFormat(long p0);
    
    public abstract boolean isImageFormat(long p0);
    
    protected boolean isURIListFormat(long n) {
        return false;
    }
    
    public SortedMap<Long, DataFlavor> getFormatsForTransferable(Transferable transferable, FlavorTable flavorTable) {
        DataFlavor[] transferDataFlavors = transferable.getTransferDataFlavors();
        if (transferDataFlavors == null) {
            return new TreeMap();
        }
        return this.getFormatsForFlavors(transferDataFlavors, flavorTable);
    }
    
    public SortedMap getFormatsForFlavor(DataFlavor dataFlavor, FlavorTable flavorTable) {
        return this.getFormatsForFlavors(new DataFlavor[] { dataFlavor }, flavorTable);
    }
    
    public SortedMap<Long, DataFlavor> getFormatsForFlavors(DataFlavor[] array, FlavorTable flavorTable) {
        HashMap hashMap = new HashMap(array.length);
        HashMap hashMap2 = new HashMap(array.length);
        HashMap hashMap3 = new HashMap(array.length);
        HashMap hashMap4 = new HashMap(array.length);
        int n = 0;
        for (int i = array.length - 1; i >= 0; --i) {
            DataFlavor that = array[i];
            if (that != null && (that.isFlavorTextType() || that.isFlavorJavaFileListType() || DataFlavor.imageFlavor.equals(that) || that.isRepresentationClassSerializable() || that.isRepresentationClassInputStream() || that.isRepresentationClassRemote())) {
                List<String> nativesForFlavor = flavorTable.getNativesForFlavor(that);
                int n2 = n + nativesForFlavor.size();
                Iterator<Object> iterator = nativesForFlavor.iterator();
                while (iterator.hasNext()) {
                    Long formatForNativeAsLong = this.getFormatForNativeAsLong((String)iterator.next());
                    Integer value = Integer.valueOf(n2--);
                    hashMap.put(formatForNativeAsLong, that);
                    hashMap3.put(formatForNativeAsLong, value);
                    if (("text".equals(that.getPrimaryType()) && "plain".equals(that.getSubType())) || that.equals(DataFlavor.stringFlavor)) {
                        hashMap2.put(formatForNativeAsLong, that);
                        hashMap4.put(formatForNativeAsLong, value);
                    }
                    else {
                        continue;
                    }
                }
                n = n2 + nativesForFlavor.size();
            }
        }
        hashMap.putAll(hashMap2);
        hashMap3.putAll(hashMap4);
        TreeMap o = new TreeMap(new DataTransferer.IndexOrderComparator(hashMap3, false));
        o.putAll(hashMap);
        return o;
    }
    
    public long[] getFormatsForTransferableAsArray(Transferable transferable, FlavorTable flavorTable) {
        return keysToLongArray(this.getFormatsForTransferable(transferable, flavorTable));
    }
    
    public long[] getFormatsForFlavorAsArray(DataFlavor dataFlavor, FlavorTable flavorTable) {
        return keysToLongArray(this.getFormatsForFlavor(dataFlavor, flavorTable));
    }
    
    public long[] getFormatsForFlavorsAsArray(DataFlavor[] array, FlavorTable flavorTable) {
        return keysToLongArray(this.getFormatsForFlavors(array, flavorTable));
    }
    
    public Map getFlavorsForFormat(long n, FlavorTable flavorTable) {
        return this.getFlavorsForFormats(new long[] { n }, flavorTable);
    }
    
    public Map getFlavorsForFormats(long[] array, FlavorTable flavorTable) {
        HashMap hashMap = new HashMap(array.length);
        HashSet set = new HashSet(array.length);
        HashSet set2 = new HashSet(array.length);
        for (int i = 0; i < array.length; ++i) {
            long l = array[i];
            for (DataFlavor that : flavorTable.getFlavorsForNative(this.getNativeForFormat(l))) {
                if (that.isFlavorTextType() || that.isFlavorJavaFileListType() || DataFlavor.imageFlavor.equals(that) || that.isRepresentationClassSerializable() || that.isRepresentationClassInputStream() || that.isRepresentationClassRemote()) {
                    Long value = Long.valueOf(l);
                    Object mapping = createMapping(value, that);
                    hashMap.put(that, value);
                    set.add(mapping);
                    set2.add(that);
                }
                else {
                    continue;
                }
            }
        }
        for (DataFlavor dataFlavor : set2) {
            Iterator<String> iterator3 = flavorTable.getNativesForFlavor(dataFlavor).iterator();
            while (iterator3.hasNext()) {
                Long formatForNativeAsLong = this.getFormatForNativeAsLong((String)iterator3.next());
                if (set.contains(createMapping(formatForNativeAsLong, dataFlavor))) {
                    hashMap.put(dataFlavor, formatForNativeAsLong);
                    break;
                }
                continue;
            }
        }
        return hashMap;
    }
    
    public Set getFlavorsForFormatsAsSet(long[] array, FlavorTable flavorTable) {
        HashSet set = new HashSet(array.length);
        for (int i = 0; i < array.length; ++i) {
            for (DataFlavor that : flavorTable.getFlavorsForNative(this.getNativeForFormat(array[i]))) {
                if (that.isFlavorTextType() || that.isFlavorJavaFileListType() || DataFlavor.imageFlavor.equals(that) || that.isRepresentationClassSerializable() || that.isRepresentationClassInputStream() || that.isRepresentationClassRemote()) {
                    set.add(that);
                }
                else {
                    continue;
                }
            }
        }
        return set;
    }
    
    public DataFlavor[] getFlavorsForFormatAsArray(long n, FlavorTable flavorTable) {
        return this.getFlavorsForFormatsAsArray(new long[] { n }, flavorTable);
    }
    
    public DataFlavor[] getFlavorsForFormatsAsArray(long[] array, FlavorTable flavorTable) {
        return setToSortedDataFlavorArray(this.getFlavorsForFormatsAsSet(array, flavorTable));
    }
    
    private static Object createMapping(Object o, Object o2) {
        return Arrays.asList(new Object[] { o, o2 });
    }
    
    protected abstract Long getFormatForNativeAsLong(String p0);
    
    protected abstract String getNativeForFormat(long p0);
    
    private String getBestCharsetForTextFormat(Long n, Transferable transferable) throws IOException {
        String s = null;
        if (transferable != null && this.isLocaleDependentTextFormat((long)n) && transferable.isDataFlavorSupported(DataTransferer.javaTextEncodingFlavor)) {
            try {
                s = new String((byte[])(byte[])transferable.getTransferData(DataTransferer.javaTextEncodingFlavor), "UTF-8");
            }
            catch (UnsupportedFlavorException ex) {}
        }
        else {
            s = this.getCharsetForTextFormat(n);
        }
        if (s == null) {
            s = getDefaultTextCharset();
            return s;
        }
        return s;
    }
    
    private byte[] translateTransferableString(String string, long l) throws IOException {
        Long value = Long.valueOf(l);
        String bestCharsetForTextFormat = this.getBestCharsetForTextFormat(value, null);
        String str = (String)DataTransferer.nativeEOLNs.get(value);
        if (str != null) {
            int length = string.length();
            StringBuffer sb = new StringBuffer(length * 2);
            for (int i = 0; i < length; ++i) {
                if (string.startsWith(str, i)) {
                    sb.append(str);
                    i += str.length() - 1;
                }
                else {
                    int char1 = string.charAt(i);
                    if (char1 == '\n') {
                        sb.append(str);
                    }
                    else {
                        sb.append((char)char1);
                    }
                }
            }
            string = sb.toString();
        }
        byte[] bytes = string.getBytes(bestCharsetForTextFormat);
        Integer n = (Integer)DataTransferer.nativeTerminators.get(value);
        if (n != null) {
            byte[] array = new byte[bytes.length + (int)n];
            System.arraycopy(bytes, 0, array, 0, bytes.length);
            for (int j = bytes.length; j < array.length; ++j) {
                array[j] = 0;
            }
            bytes = array;
            return bytes;
        }
        return bytes;
    }
    
    private String translateBytesToString(byte[] bytes, long l, Transferable transferable) throws IOException {
        Long value = Long.valueOf(l);
        String bestCharsetForTextFormat = this.getBestCharsetForTextFormat(value, transferable);
        String s = (String)DataTransferer.nativeEOLNs.get(value);
        Integer n = (Integer)DataTransferer.nativeTerminators.get(value);
        int i;
        if (n != null) {
            int intValue = (int)n;
            i = 0;
        Label_0061:
            while (i < bytes.length - intValue + 1) {
                while (i < i + intValue) {
                    if (bytes[i] != 0) {
                        i += intValue;
                        continue Label_0061;
                    }
                    ++i;
                }
                break;
            }
        }
        else {
            i = bytes.length;
        }
        String o = new String(bytes, 0, i, bestCharsetForTextFormat);
        if (s != null) {
            char[] charArray = o.toCharArray();
            char[] charArray2 = s.toCharArray();
            int count = 0;
            int j = 0;
            while (j < charArray.length) {
                if (j + charArray2.length > charArray.length) {
                    charArray[count++] = charArray[j++];
                }
                else {
                    boolean n2 = true;
                    int k = 0;
                    while (k < charArray2.length) {
                        if (charArray2[k] != charArray[j]) {
                            n2 = false;
                            break;
                        }
                        ++k;
                        ++j;
                    }
                    if (!(n2 == 0)) {
                        charArray[count++] = '\n';
                        j += charArray2.length;
                    }
                    else {
                        charArray[count++] = charArray[j++];
                    }
                }
            }
            o = new String(charArray, 0, count);
            return o;
        }
        return o;
    }
    
    public byte[] translateTransferable(Transferable transferable, DataFlavor that, long n) throws IOException {
        Object o;
        int n2;
        try {
            o = transferable.getTransferData(that);
            if (o == null) {
                return null;
            }
            if (that.equals(DataFlavor.plainTextFlavor) && !(o instanceof InputStream)) {
                o = transferable.getTransferData(DataFlavor.stringFlavor);
                if (o == null) {
                    return null;
                }
                n2 = true;
            }
            else {
                n2 = false;
            }
        }
        catch (UnsupportedFlavorException ex) {
            throw new IOException(ex.getMessage());
        }
        if (n2 || (String.class.equals(that.getRepresentationClass()) && isFlavorCharsetTextType(that) && this.isTextFormat(n))) {
            return this.translateTransferableString(this.removeSuspectedData(that, transferable, (String)o), n);
        }
        if (that.isRepresentationClassReader()) {
            if (!isFlavorCharsetTextType(that) || !this.isTextFormat(n)) {
                throw new IOException("cannot transfer non-text data as Reader");
            }
            StringBuffer o5 = new StringBuffer();
            try (final Reader reader = (Reader)o) {
                int read2;
                while ((read2 = reader.read()) != -1) {
                    o5.append((char)read2);
                }
            }
            return this.translateTransferableString(o5.toString(), n);
        }
        else if (that.isRepresentationClassCharBuffer()) {
            if (!isFlavorCharsetTextType(that) || !this.isTextFormat(n)) {
                throw new IOException("cannot transfer non-text data as CharBuffer");
            }
            CharBuffer charBuffer = (CharBuffer)o;
            int remaining2 = charBuffer.remaining();
            char[] array4 = new char[remaining2];
            charBuffer.get(array4, 0, remaining2);
            return this.translateTransferableString(new String(array4), n);
        }
        else if (char[].class.equals(that.getRepresentationClass())) {
            if (!isFlavorCharsetTextType(that) || !this.isTextFormat(n)) {
                throw new IOException("cannot transfer non-text data as char array");
            }
            return this.translateTransferableString(new String((char[])(char[])o), n);
        }
        else if (that.isRepresentationClassByteBuffer()) {
            ByteBuffer byteBuffer = (ByteBuffer)o;
            int remaining = byteBuffer.remaining();
            byte[] array3 = new byte[remaining];
            byteBuffer.get(array3, 0, remaining);
            if (isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                return this.translateTransferableString(new String(array3, getTextCharset(that)), n);
            }
            return array3;
        }
        else if (byte[].class.equals(that.getRepresentationClass())) {
            byte[] bytes3 = (byte[])(byte[])o;
            if (isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                return this.translateTransferableString(new String(bytes3, getTextCharset(that)), n);
            }
            return bytes3;
        }
        else if (DataFlavor.imageFlavor.equals(that)) {
            if (!this.isImageFormat(n)) {
                throw new IOException("Data translation failed: not an image format");
            }
            byte[] imageToPlatformBytes = this.imageToPlatformBytes((Image)o, n);
            if (imageToPlatformBytes == null) {
                throw new IOException("Data translation failed: cannot convert java image to native format");
            }
            return imageToPlatformBytes;
        }
        else {
            byte[] array = null;
            if (this.isFileFormat(n)) {
                if (!DataFlavor.javaFileListFlavor.equals(that)) {
                    throw new IOException("data translation failed");
                }
                try (final ByteArrayOutputStream convertFileListToBytes = this.convertFileListToBytes(this.castToFiles((List)o, getUserProtectionDomain(transferable)))) {
                    array = convertFileListToBytes.toByteArray();
                }
            }
            else if (this.isURIListFormat(n)) {
                if (!DataFlavor.javaFileListFlavor.equals(that)) {
                    throw new IOException("data translation failed");
                }
                String nativeForFormat = this.getNativeForFormat(n);
                String parameter = null;
                if (nativeForFormat != null) {
                    try {
                        parameter = new DataFlavor(nativeForFormat).getParameter("charset");
                    }
                    catch (ClassNotFoundException cause) {
                        throw new IOException(cause);
                    }
                }
                if (parameter == null) {
                    parameter = "UTF-8";
                }
                ArrayList<String> castToFiles = this.castToFiles((List)o, getUserProtectionDomain(transferable));
                ArrayList o3 = new ArrayList(castToFiles.size());
                Iterator<Object> iterator = castToFiles.iterator();
                while (iterator.hasNext()) {
                    URI uri = new File((String)iterator.next()).toURI();
                    try {
                        o3.add(new URI(uri.getScheme(), "", uri.getPath(), uri.getFragment()).toString());
                    }
                    catch (URISyntaxException cause2) {
                        throw new IOException(cause2);
                    }
                }
                byte[] bytes = "\r\n".getBytes(parameter);
                try (final Object o4 = new ByteArrayOutputStream()) {
                    for (int j = 0; j < o3.size(); ++j) {
                        byte[] bytes2 = ((String)o3.get(j)).getBytes(parameter);
                        o4.write(bytes2, 0, bytes2.length);
                        o4.write(bytes, 0, bytes.length);
                    }
                    array = o4.toByteArray();
                }
            }
            else if (that.isRepresentationClassInputStream()) {
                if (!(o instanceof InputStream)) {
                    return new byte[0];
                }
                try (final Object o2 = new ByteArrayOutputStream()) {
                    try (final InputStream inputStream = (InputStream)o) {
                        int available = inputStream.available();
                        byte[] array2 = new byte[(available > 8192) ? available : 8192];
                        int i;
                        do {
                            int read;
                            if ((i = ((read = inputStream.read(array2, 0, array2.length)) == -1)) == 0) {
                                o2.write(array2, 0, read);
                            }
                            else {
                                continue;
                            }
                        } while (i == 0);
                    }
                    if (isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                        return this.translateTransferableString(new String(o2.toByteArray(), getTextCharset(that)), n);
                    }
                    else {
                        array = o2.toByteArray();
                    }
                }
            }
            else if (that.isRepresentationClassRemote()) {
                array = convertObjectToBytes(DataTransferer.RMI.newMarshalledObject(o));
            }
            else if (that.isRepresentationClassSerializable()) {
                array = convertObjectToBytes(o);
            }
            else {
                throw new IOException("data translation failed");
            }
            return array;
        }
    }
    
    private static byte[] convertObjectToBytes(Object obj) throws IOException {
        try (final ByteArrayOutputStream out = new ByteArrayOutputStream();
             final ObjectOutputStream objectOutputStream = new ObjectOutputStream(out)) {
            objectOutputStream.writeObject(obj);
            return out.toByteArray();
        }
    }
    
    protected abstract ByteArrayOutputStream convertFileListToBytes(ArrayList<String> p0) throws IOException;
    
    private String removeSuspectedData(DataFlavor dataFlavor, Transferable transferable, final String s) throws IOException {
        if (null == System.getSecurityManager() || !dataFlavor.isMimeTypeEqual("text/uri-list")) {
            return s;
        }
        final ProtectionDomain userProtectionDomain = getUserProtectionDomain(transferable);
        String s2;
        try {
            s2 = (String)AccessController.doPrivileged(new PrivilegedExceptionAction() {
                @Override
                public Object run() {
                    StringBuffer sb = new StringBuffer(s.length());
                    String[] split;
                    for (String s : split = s.split("(\\s)+")) {
                        File o = new File(s);
                        if (o.exists() && !DataTransferer.access$100(o) && !DataTransferer.access$200(DataTransferer.this, o, userProtectionDomain)) {
                            if (0 != sb.length()) {
                                sb.append("\\r\\n");
                            }
                            sb.append(s);
                        }
                    }
                    return sb.toString();
                }
            });
        }
        catch (PrivilegedActionException cause) {
            throw new IOException(cause.getMessage(), cause);
        }
        return s2;
    }
    
    private static ProtectionDomain getUserProtectionDomain(Transferable transferable) {
        return transferable.getClass().getProtectionDomain();
    }
    
    private boolean isForbiddenToRead(File file, ProtectionDomain protectionDomain) {
        if (null == protectionDomain) {
            return false;
        }
        try {
            if (protectionDomain.implies(new FilePermission(file.getCanonicalPath(), "read, delete"))) {
                return false;
            }
        }
        catch (IOException ex) {}
        return true;
    }
    
    private ArrayList<String> castToFiles(final List list, final ProtectionDomain protectionDomain) throws IOException {
        final ArrayList<Object> list2 = new ArrayList();
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction() {
                @Override
                public Object run() throws IOException {
                    Iterator<Object> iterator = list.iterator();
                    while (iterator.hasNext()) {
                        File access$300 = DataTransferer.access$300(DataTransferer.this, iterator.next());
                        if (access$300 != null && (null == System.getSecurityManager() || (!DataTransferer.access$100(access$300) && !DataTransferer.access$200(DataTransferer.this, access$300, protectionDomain)))) {
                            list2.add(access$300.getCanonicalPath());
                        }
                        else {
                            continue;
                        }
                    }
                    return null;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw new IOException(ex.getMessage());
        }
        return list2;
    }
    
    private File castToFile(Object o) throws IOException {
        String canonicalPath;
        if ((o instanceof File)) {
            canonicalPath = ((File)o).getCanonicalPath();
        }
        else if ((o instanceof String)) {
            canonicalPath = (String)o;
        }
        else {
            return null;
        }
        return new File(canonicalPath);
    }
    
    private static boolean isFileInWebstartedCache(File parentFile) {
        if (DataTransferer.deploymentCacheDirectoryList.isEmpty()) {
            String[] deployment_CACHE_PROPERTIES = DataTransferer.DEPLOYMENT_CACHE_PROPERTIES;
            for (int length = deployment_CACHE_PROPERTIES.length, i = 0; i < length; ++i) {
                String property = System.getProperty(deployment_CACHE_PROPERTIES[i]);
                if (property != null) {
                    try {
                        File canonicalFile = new File(property).getCanonicalFile();
                        if (canonicalFile != null) {
                            DataTransferer.deploymentCacheDirectoryList.add(canonicalFile);
                        }
                    }
                    catch (IOException ex) {}
                }
            }
        }
        for (File obj : DataTransferer.deploymentCacheDirectoryList) {
            while (parentFile != null) {
                if (parentFile.equals(obj)) {
                    return true;
                }
                parentFile = parentFile.getParentFile();
            }
        }
        return false;
    }
    
    public Object translateBytes(byte[] bytes, DataFlavor that, long n, Transferable transferable) throws IOException {
        Object o = null;
        if (this.isFileFormat(n)) {
            if (!DataFlavor.javaFileListFlavor.equals(that)) {
                throw new IOException("data translation failed");
            }
            String[] dragQueryFile = this.dragQueryFile(bytes);
            if (dragQueryFile == null) {
                return null;
            }
            File[] a = new File[dragQueryFile.length];
            for (int j = 0; j < dragQueryFile.length; ++j) {
                a[j] = new File(dragQueryFile[j]);
            }
            o = Arrays.asList(a);
        }
        else if (this.isURIListFormat(n) && DataFlavor.javaFileListFlavor.equals(that)) {
            try (final ByteArrayInputStream byteArrayInputStream4 = new ByteArrayInputStream(bytes)) {
                URI[] dragQueryURIs = this.dragQueryURIs(byteArrayInputStream4, n, transferable);
                if (dragQueryURIs == null) {
                    return null;
                }
                else {
                    ArrayList<File> list = new ArrayList();
                    for (URI uri : dragQueryURIs) {
                        try {
                            list.add(new File(uri));
                        }
                        catch (IllegalArgumentException ex2) {}
                    }
                    o = list;
                }
            }
        }
        else if (String.class.equals(that.getRepresentationClass()) && isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
            o = this.translateBytesToString(bytes, n, transferable);
        }
        else if (that.isRepresentationClassReader()) {
            try (final ByteArrayInputStream byteArrayInputStream3 = new ByteArrayInputStream(bytes)) {
                o = this.translateStream(byteArrayInputStream3, that, n, transferable);
            }
        }
        else if (that.isRepresentationClassCharBuffer()) {
            if (!isFlavorCharsetTextType(that) || !this.isTextFormat(n)) {
                throw new IOException("cannot transfer non-text data as CharBuffer");
            }
            o = this.constructFlavoredObject(CharBuffer.wrap(this.translateBytesToString(bytes, n, transferable)), that, CharBuffer.class);
        }
        else if (char[].class.equals(that.getRepresentationClass())) {
            if (!isFlavorCharsetTextType(that) || !this.isTextFormat(n)) {
                throw new IOException("cannot transfer non-text data as char array");
            }
            o = this.translateBytesToString(bytes, n, transferable).toCharArray();
        }
        else if (that.isRepresentationClassByteBuffer()) {
            if (isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                bytes = this.translateBytesToString(bytes, n, transferable).getBytes(getTextCharset(that));
            }
            o = this.constructFlavoredObject(ByteBuffer.wrap(bytes), that, ByteBuffer.class);
        }
        else if (byte[].class.equals(that.getRepresentationClass())) {
            if (isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                o = this.translateBytesToString(bytes, n, transferable).getBytes(getTextCharset(that));
            }
            else {
                o = bytes;
            }
        }
        else if (that.isRepresentationClassInputStream()) {
            try (final ByteArrayInputStream byteArrayInputStream2 = new ByteArrayInputStream(bytes)) {
                o = this.translateStream(byteArrayInputStream2, that, n, transferable);
            }
        }
        else if (that.isRepresentationClassRemote()) {
            try (final ByteArrayInputStream in = new ByteArrayInputStream(bytes);
                 final ObjectInputStream objectInputStream = new ObjectInputStream(in)) {
                o = DataTransferer.RMI.getMarshalledObject(objectInputStream.readObject());
            }
            catch (Exception ex) {
                throw new IOException(ex.getMessage());
            }
        }
        else if (that.isRepresentationClassSerializable()) {
            try (final ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes)) {
                o = this.translateStream(byteArrayInputStream, that, n, transferable);
            }
        }
        else if (DataFlavor.imageFlavor.equals(that)) {
            if (!this.isImageFormat(n)) {
                throw new IOException("data translation failed");
            }
            o = this.platformImageBytesToImage(bytes, n);
        }
        if (o == null) {
            throw new IOException("data translation failed");
        }
        return o;
    }
    
    public Object translateStream(InputStream inputStream, DataFlavor that, long n, Transferable transferable) throws IOException {
        Object o = null;
        if (this.isURIListFormat(n) && DataFlavor.javaFileListFlavor.equals(that)) {
            URI[] dragQueryURIs = this.dragQueryURIs(inputStream, n, transferable);
            if (dragQueryURIs == null) {
                return null;
            }
            ArrayList<File> list = new ArrayList();
            for (URI uri : dragQueryURIs) {
                try {
                    list.add(new File(uri));
                }
                catch (IllegalArgumentException ex3) {}
            }
            o = list;
        }
        else {
            if (String.class.equals(that.getRepresentationClass()) && isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                return this.translateBytesToString(inputStreamToByteArray(inputStream), n, transferable);
            }
            if (DataFlavor.plainTextFlavor.equals(that)) {
                o = new StringReader(this.translateBytesToString(inputStreamToByteArray(inputStream), n, transferable));
            }
            else if (that.isRepresentationClassInputStream()) {
                o = this.translateStreamToInputStream(inputStream, that, n, transferable);
            }
            else if (that.isRepresentationClassReader()) {
                if (!isFlavorCharsetTextType(that) || !this.isTextFormat(n)) {
                    throw new IOException("cannot transfer non-text data as Reader");
                }
                o = this.constructFlavoredObject(new InputStreamReader((InputStream)this.translateStreamToInputStream(inputStream, DataFlavor.plainTextFlavor, n, transferable), getTextCharset(DataFlavor.plainTextFlavor)), that, Reader.class);
            }
            else if (byte[].class.equals(that.getRepresentationClass())) {
                if (isFlavorCharsetTextType(that) && this.isTextFormat(n)) {
                    o = this.translateBytesToString(inputStreamToByteArray(inputStream), n, transferable).getBytes(getTextCharset(that));
                }
                else {
                    o = inputStreamToByteArray(inputStream);
                }
            }
            else if (that.isRepresentationClassRemote()) {
                try (final ObjectInputStream objectInputStream2 = new ObjectInputStream(inputStream)) {
                    o = DataTransferer.RMI.getMarshalledObject(objectInputStream2.readObject());
                }
                catch (Exception ex2) {
                    throw new IOException(ex2.getMessage());
                }
            }
            else if (that.isRepresentationClassSerializable()) {
                try (final ObjectInputStream objectInputStream = new ObjectInputStream(inputStream)) {
                    o = objectInputStream.readObject();
                }
                catch (Exception ex) {
                    throw new IOException(ex.getMessage());
                }
            }
            else if (DataFlavor.imageFlavor.equals(that)) {
                if (!this.isImageFormat(n)) {
                    throw new IOException("data translation failed");
                }
                o = this.platformImageBytesToImage(inputStreamToByteArray(inputStream), n);
            }
        }
        if (o == null) {
            throw new IOException("data translation failed");
        }
        return o;
    }
    
    private Object translateStreamToInputStream(InputStream inputStream, DataFlavor dataFlavor, long n, Transferable transferable) throws IOException {
        if (isFlavorCharsetTextType(dataFlavor) && this.isTextFormat(n)) {
            inputStream = new DataTransferer.ReencodingInputStream(inputStream, n, getTextCharset(dataFlavor), transferable);
        }
        return this.constructFlavoredObject(inputStream, dataFlavor, InputStream.class);
    }
    
    private Object constructFlavoredObject(Object o, DataFlavor dataFlavor, Class obj) throws IOException {
        final Class<Object> representationClass = dataFlavor.getRepresentationClass();
        if (obj.equals(representationClass)) {
            return o;
        }
        Constructor[] array;
        try {
            array = (Constructor[])(Constructor[])AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    return representationClass.getConstructors();
                }
            });
        }
        catch (SecurityException ex) {
            throw new IOException(ex.getMessage());
        }
        Constructor<Object> constructor = null;
        for (int i = 0; i < array.length; ++i) {
            if (Modifier.isPublic(array[i].getModifiers())) {
                Class[] parameterTypes = array[i].getParameterTypes();
                if (parameterTypes != null && parameterTypes.length == 1 && obj.equals(parameterTypes[0])) {
                    constructor = array[i];
                    break;
                }
            }
        }
        if (constructor == null) {
            throw new IOException("can't find <init>(L" + obj + ";)V for class: " + representationClass.getName());
        }
        try {
            return constructor.newInstance(new Object[] { o });
        }
        catch (Exception ex2) {
            throw new IOException(ex2.getMessage());
        }
    }
    
    protected abstract String[] dragQueryFile(byte[] p0);
    
    protected URI[] dragQueryURIs(InputStream inputStream, long n, Transferable transferable) throws IOException {
        throw new IOException(new UnsupportedOperationException("not implemented on this platform"));
    }
    
    protected abstract Image platformImageBytesToImage(byte[] p0, long p1) throws IOException;
    
    protected Image standardImageBytesToImage(byte[] buf, String str) throws IOException {
        Iterator<ImageReader> imageReadersByMIMEType = ImageIO.getImageReadersByMIMEType(str);
        if (!imageReadersByMIMEType.hasNext()) {
            throw new IOException("No registered service provider can decode  an image from " + str);
        }
        Object o = null;
        while (imageReadersByMIMEType.hasNext()) {
            ImageReader imageReader = (ImageReader)imageReadersByMIMEType.next();
            try (final Object input = new ByteArrayInputStream(buf)) {
                ImageInputStream imageInputStream = ImageIO.createImageInputStream(input);
                try {
                    ImageReadParam defaultReadParam = imageReader.getDefaultReadParam();
                    imageReader.setInput(imageInputStream, true, true);
                    BufferedImage read = imageReader.read(imageReader.getMinIndex(), defaultReadParam);
                    if (read != null) {
                        imageInputStream.close();
                        imageReader.dispose();
                        return read;
                    }
                }
                finally {
                    imageInputStream.close();
                    imageReader.dispose();
                }
                if (input != null) {
                    Throwable t;
                    if (t != null) {
                        try {
                            input.close();
                        }
                        catch (Throwable exception2) {
                            t.addSuppressed(exception2);
                        }
                    }
                    else {
                        input.close();
                    }
                }
                else {
                    continue;
                }
            }
            catch (IOException ex) {
                o = ex;
            }
        }
        if (o == null) {
            o = new IOException("Registered service providers failed to decode an image from " + str);
        }
        throw o;
    }
    
    protected abstract byte[] imageToPlatformBytes(Image p0, long p1) throws IOException;
    
    protected byte[] imageToStandardBytes(Image image, String s) throws IOException {
        IOException ex = null;
        if (!ImageIO.getImageWritersByMIMEType(s).hasNext()) {
            throw new IOException("No registered service provider can encode  an image to " + s);
        }
        if ((image instanceof RenderedImage)) {
            try {
                return this.imageToStandardBytesImpl((RenderedImage)image, s);
            }
            catch (IOException ex2) {
                ex = ex2;
            }
        }
        int w;
        int h;
        if ((image instanceof ToolkitImage)) {
            ImageRepresentation imageRep = ((ToolkitImage)image).getImageRep();
            imageRep.reconstruct(32);
            w = imageRep.getWidth();
            h = imageRep.getHeight();
        }
        else {
            w = image.getWidth(null);
            h = image.getHeight(null);
        }
        ColorModel rgBdefault = ColorModel.getRGBdefault();
        BufferedImage o = new BufferedImage(rgBdefault, rgBdefault.createCompatibleWritableRaster(w, h), rgBdefault.isAlphaPremultiplied(), null);
        Graphics graphics = o.getGraphics();
        try {
            graphics.drawImage(image, 0, 0, w, h, null);
        }
        finally {
            graphics.dispose();
        }
        try {
            return this.imageToStandardBytesImpl(o, s);
        }
        catch (IOException ex3) {
            if (ex != null) {
                throw ex;
            }
        }
    }
    
    protected byte[] imageToStandardBytesImpl(RenderedImage obj, String s) throws IOException {
        Iterator<ImageWriter> imageWritersByMIMEType = ImageIO.getImageWritersByMIMEType(s);
        ImageTypeSpecifier imageTypeSpecifier = new ImageTypeSpecifier(obj);
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        Object o = null;
        while (imageWritersByMIMEType.hasNext()) {
            ImageWriter imageWriter = (ImageWriter)imageWritersByMIMEType.next();
            if (!imageWriter.getOriginatingProvider().canEncodeImage(imageTypeSpecifier)) {
                continue;
            }
            try {
                ImageOutputStream imageOutputStream = ImageIO.createImageOutputStream(output);
                try {
                    imageWriter.setOutput(imageOutputStream);
                    imageWriter.write(obj);
                    imageOutputStream.flush();
                }
                finally {
                    imageOutputStream.close();
                }
            }
            catch (IOException ex) {
                imageWriter.dispose();
                output.reset();
                o = ex;
                continue;
            }
            imageWriter.dispose();
            output.close();
            return output.toByteArray();
        }
        output.close();
        if (o == null) {
            o = new IOException("Registered service providers failed to encode " + obj + " to " + s);
        }
        throw o;
    }
    
    private Object concatData(Object o, Object o2) {
        ByteArrayInputStream s1;
        InputStream s2;
        if ((o instanceof byte[])) {
            byte[] buf = (byte[])(byte[])o;
            if ((o2 instanceof byte[])) {
                byte[] array = (byte[])(byte[])o2;
                byte[] array2 = new byte[buf.length + array.length];
                System.arraycopy(buf, 0, array2, 0, buf.length);
                System.arraycopy(array, 0, array2, buf.length, array.length);
                return array2;
            }
            s1 = new ByteArrayInputStream(buf);
            s2 = (InputStream)o2;
        }
        else {
            s1 = (InputStream)o;
            if ((o2 instanceof byte[])) {
                s2 = new ByteArrayInputStream((byte[])(byte[])o2);
            }
            else {
                s2 = (InputStream)o2;
            }
        }
        return new SequenceInputStream(s1, s2);
    }
    
    public byte[] convertData(Object o, final Transferable transferable, final long l, final Map map, boolean b) throws IOException {
        byte[] translateTransferable = null;
        if (b) {
            try {
                final Stack<Object> stack = new Stack();
                Runnable runnable = new Runnable() {
                    private boolean done = false;
                    
                    @Override
                    public void run() {
                        if (this.done) {
                            return;
                        }
                        byte[] translateTransferable = null;
                        try {
                            DataFlavor dataFlavor = (DataFlavor)map.get(Long.valueOf(l));
                            if (dataFlavor != null) {
                                translateTransferable = DataTransferer.this.translateTransferable(transferable, dataFlavor, l);
                            }
                        }
                        catch (Exception ex) {
                            ex.printStackTrace();
                            translateTransferable = null;
                        }
                        try {
                            DataTransferer.this.getToolkitThreadBlockedHandler().lock();
                            stack.push(translateTransferable);
                            DataTransferer.this.getToolkitThreadBlockedHandler().exit();
                        }
                        finally {
                            DataTransferer.this.getToolkitThreadBlockedHandler().unlock();
                            this.done = true;
                        }
                        return;
                    }
                };
                AppContext targetToAppContext = SunToolkit.targetToAppContext(o);
                this.getToolkitThreadBlockedHandler().lock();
                if (targetToAppContext != null) {
                    targetToAppContext.put("DATA_CONVERTER_KEY", runnable);
                }
                SunToolkit.executeOnEventHandlerThread(o, runnable);
                while (stack.empty()) {
                    this.getToolkitThreadBlockedHandler().enter();
                }
                if (targetToAppContext != null) {
                    targetToAppContext.remove("DATA_CONVERTER_KEY");
                }
                translateTransferable = (byte[])(byte[])stack.pop();
            }
            finally {
                this.getToolkitThreadBlockedHandler().unlock();
            }
        }
        else {
            DataFlavor dataFlavor = (DataFlavor)map.get(Long.valueOf(l));
            if (dataFlavor != null) {
                translateTransferable = this.translateTransferable(transferable, dataFlavor, l);
            }
        }
        return translateTransferable;
    }
    
    public void processDataConversionRequests() {
        if (EventQueue.isDispatchThread()) {
            AppContext appContext = AppContext.getAppContext();
            this.getToolkitThreadBlockedHandler().lock();
            try {
                Runnable runnable = (Runnable)appContext.get("DATA_CONVERTER_KEY");
                if (runnable != null) {
                    runnable.run();
                    appContext.remove("DATA_CONVERTER_KEY");
                }
            }
            finally {
                this.getToolkitThreadBlockedHandler().unlock();
            }
        }
    }
    
    public abstract ToolkitThreadBlockedHandler getToolkitThreadBlockedHandler();
    
    public static long[] keysToLongArray(SortedMap sortedMap) {
        Set<Object> keySet = sortedMap.keySet();
        long[] array = new long[keySet.size()];
        int n = 0;
        Iterator<Object> iterator = keySet.iterator();
        while (iterator.hasNext()) {
            array[n] = (long)iterator.next();
            ++n;
        }
        return array;
    }
    
    public static DataFlavor[] setToSortedDataFlavorArray(Set set) {
        DataFlavor[] a = new DataFlavor[set.size()];
        set.toArray(a);
        Arrays.sort(a, new DataTransferer.DataFlavorComparator(false));
        return a;
    }
    
    protected static byte[] inputStreamToByteArray(InputStream inputStream) throws IOException {
        try (final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream()) {
            byte[] array = new byte[8192];
            int read;
            while ((read = inputStream.read(array)) != -1) {
                byteArrayOutputStream.write(array, 0, read);
            }
            return byteArrayOutputStream.toByteArray();
        }
    }
    
    public LinkedHashSet<DataFlavor> getPlatformMappingsForNative(String s) {
        return new LinkedHashSet();
    }
    
    public LinkedHashSet<String> getPlatformMappingsForFlavor(DataFlavor dataFlavor) {
        return new LinkedHashSet();
    }
    
    static boolean access$100(File file) {
        return isFileInWebstartedCache(file);
    }
    
    static boolean access$200(DataTransferer dataTransferer, File file, ProtectionDomain protectionDomain) {
        return dataTransferer.isForbiddenToRead(file, protectionDomain);
    }
    
    static File access$300(DataTransferer dataTransferer, Object o) throws IOException {
        return dataTransferer.castToFile(o);
    }
    
    static {
        textNatives = Collections.synchronizedSet(new HashSet());
        nativeCharsets = Collections.synchronizedMap(new HashMap());
        nativeEOLNs = Collections.synchronizedMap(new HashMap());
        nativeTerminators = Collections.synchronizedMap(new HashMap());
        dtLog = PlatformLogger.getLogger("sun.awt.datatransfer.DataTransfer");
        DataFlavor plainTextStringFlavor2 = null;
        try {
            plainTextStringFlavor2 = new DataFlavor("text/plain;charset=Unicode;class=java.lang.String");
        }
        catch (ClassNotFoundException ex) {}
        plainTextStringFlavor = plainTextStringFlavor2;
        Object javaTextEncodingFlavor2 = null;
        try {
            javaTextEncodingFlavor2 = new DataFlavor("application/x-java-text-encoding;class=\"[B\"");
        }
        catch (ClassNotFoundException ex2) {}
        javaTextEncodingFlavor = javaTextEncodingFlavor2;
        HashMap m = new HashMap(17);
        m.put("sgml", Boolean.TRUE);
        m.put("xml", Boolean.TRUE);
        m.put("html", Boolean.TRUE);
        m.put("enriched", Boolean.TRUE);
        m.put("richtext", Boolean.TRUE);
        m.put("uri-list", Boolean.TRUE);
        m.put("directory", Boolean.TRUE);
        m.put("css", Boolean.TRUE);
        m.put("calendar", Boolean.TRUE);
        m.put("plain", Boolean.TRUE);
        m.put("rtf", Boolean.FALSE);
        m.put("tab-separated-values", Boolean.FALSE);
        m.put("t140", Boolean.FALSE);
        m.put("rfc822-headers", Boolean.FALSE);
        m.put("parityfec", Boolean.FALSE);
        textMIMESubtypeCharsetSupport = Collections.synchronizedMap(m);
        DEPLOYMENT_CACHE_PROPERTIES = new String[] { "deployment.system.cachedir", "deployment.user.cachedir", "deployment.javaws.cachedir", "deployment.javapi.cachedir" };
        deploymentCacheDirectoryList = new ArrayList();
    }
    
    private static class StandardEncodingsHolder
    {
        private static final SortedSet<String> standardEncodings;
        
        private static SortedSet<String> load() {
            TreeSet<String> s = new TreeSet(new DataTransferer.CharsetComparator(false));
            s.add("US-ASCII");
            s.add("ISO-8859-1");
            s.add("UTF-8");
            s.add("UTF-16BE");
            s.add("UTF-16LE");
            s.add("UTF-16");
            s.add(DataTransferer.getDefaultTextCharset());
            return Collections.unmodifiableSortedSet(s);
        }
        
        static {
            standardEncodings = load();
        }
    }
    
    public class ReencodingInputStream extends InputStream
    {
        protected BufferedReader wrapped;
        protected final char[] in;
        protected byte[] out;
        protected CharsetEncoder encoder;
        protected CharBuffer inBuf;
        protected ByteBuffer outBuf;
        protected char[] eoln;
        protected int numTerminators;
        protected boolean eos;
        protected int index;
        protected int limit;
        
        public ReencodingInputStream(InputStream in, long l, String charsetName, Transferable transferable) throws IOException {
            this.in = new char[2];
            Long value = Long.valueOf(l);
            String charsetName2 = null;
            if (DataTransferer.this.isLocaleDependentTextFormat(l) && transferable != null && transferable.isDataFlavorSupported(DataTransferer.javaTextEncodingFlavor)) {
                try {
                    charsetName2 = new String((byte[])(byte[])transferable.getTransferData(DataTransferer.javaTextEncodingFlavor), "UTF-8");
                }
                catch (UnsupportedFlavorException ex) {}
            }
            else {
                charsetName2 = DataTransferer.this.getCharsetForTextFormat(value);
            }
            if (charsetName2 == null) {
                charsetName2 = DataTransferer.getDefaultTextCharset();
            }
            this.wrapped = new BufferedReader(new InputStreamReader(in, charsetName2));
            if (charsetName == null) {
                throw new NullPointerException("null target encoding");
            }
            try {
                this.encoder = Charset.forName(charsetName).newEncoder();
                this.out = new byte[(int)((double)(this.encoder.maxBytesPerChar() * 2.0f) + 0.5)];
                this.inBuf = CharBuffer.wrap(this.in);
                this.outBuf = ByteBuffer.wrap(this.out);
            }
            catch (IllegalCharsetNameException ex2) {
                throw new IOException(ex2.toString());
            }
            catch (UnsupportedCharsetException ex3) {
                throw new IOException(ex3.toString());
            }
            catch (UnsupportedOperationException ex4) {
                throw new IOException(ex4.toString());
            }
            String s = (String)DataTransferer.nativeEOLNs.get(value);
            if (s != null) {
                this.eoln = s.toCharArray();
            }
            Integer n = (Integer)DataTransferer.nativeTerminators.get(value);
            if (n != null) {
                this.numTerminators = (int)n;
            }
            return;
        }
        
        private int readChar() throws IOException {
            int read = this.wrapped.read();
            if (read == -1) {
                this.eos = true;
                return -1;
            }
            if (this.numTerminators > 0 && read == 0) {
                this.eos = true;
                return -1;
            }
            if (this.eoln != null && this.matchCharArray(this.eoln, read)) {
                read = 10;
            }
            return read;
        }
        
        @Override
        public int read() throws IOException {
            if (this.eos) {
                return -1;
            }
            if (this.index >= this.limit) {
                int char1 = this.readChar();
                if (char1 == -1) {
                    return -1;
                }
                this.in[0] = (char)char1;
                this.in[1] = '\0';
                this.inBuf.limit(1);
                if (Character.isHighSurrogate((char)char1)) {
                    int char2 = this.readChar();
                    if (char2 != -1) {
                        this.in[1] = (char)char2;
                        this.inBuf.limit(2);
                    }
                }
                this.inBuf.rewind();
                this.outBuf.limit(this.out.length).rewind();
                this.encoder.encode(this.inBuf, this.outBuf, false);
                this.outBuf.flip();
                this.limit = this.outBuf.limit();
                this.index = 0;
                return this.read();
            }
            else {
                return this.out[this.index++] & 0xFF;
            }
        }
        
        @Override
        public int available() throws IOException {
            return this.eos ? 0 : (this.limit - this.index);
        }
        
        @Override
        public void close() throws IOException {
            this.wrapped.close();
        }
        
        private boolean matchCharArray(char[] array, int read) throws IOException {
            this.wrapped.mark(array.length);
            int i = 0;
            if ((char)read == array[0]) {
                i = 1;
                while (i < array.length) {
                    read = this.wrapped.read();
                    if (read != -1) {
                        if ((char)read != array[i]) {
                            break;
                        }
                        ++i;
                    }
                    else {
                        break;
                    }
                }
            }
            if (i == array.length) {
                return true;
            }
            this.wrapped.reset();
            return false;
        }
    }
    
    public abstract static class IndexedComparator implements Comparator
    {
        public static final boolean SELECT_BEST = true;
        public static final boolean SELECT_WORST = false;
        protected final boolean order;
        
        public IndexedComparator() {
            this(true);
        }
        
        public IndexedComparator(boolean order) {
            this.order = order;
        }
        
        protected static int compareIndices(Map map, Object o, Object o2, Integer n) {
            Integer n2 = (Integer)map.get(o);
            Integer anotherInteger = (Integer)map.get(o2);
            if (n2 == null) {
                n2 = n;
            }
            if (anotherInteger == null) {
                anotherInteger = n;
            }
            return n2.compareTo(anotherInteger);
        }
        
        protected static int compareLongs(Map map, Object o, Object o2, Long n) {
            Long n2 = (Long)map.get(o);
            Long anotherLong = (Long)map.get(o2);
            if (n2 == null) {
                n2 = n;
            }
            if (anotherLong == null) {
                anotherLong = n;
            }
            return n2.compareTo(anotherLong);
        }
    }
    
    public static class CharsetComparator extends DataTransferer.IndexedComparator
    {
        private static final Map charsets;
        private static String defaultEncoding;
        private static final Integer DEFAULT_CHARSET_INDEX;
        private static final Integer OTHER_CHARSET_INDEX;
        private static final Integer WORST_CHARSET_INDEX;
        private static final Integer UNSUPPORTED_CHARSET_INDEX;
        private static final String UNSUPPORTED_CHARSET = "UNSUPPORTED";
        
        public CharsetComparator() {
            this(true);
        }
        
        public CharsetComparator(boolean b) {
            super(b);
        }
        
        @Override
        public int compare(Object o, Object o2) {
            String s;
            String s2;
            if (this.order) {
                s = (String)o;
                s2 = (String)o2;
            }
            else {
                s = (String)o2;
                s2 = (String)o;
            }
            return this.compareCharsets(s, s2);
        }
        
        protected int compareCharsets(String encoding, String encoding2) {
            encoding = getEncoding(encoding);
            encoding2 = getEncoding(encoding2);
            int compareIndices = DataTransferer.IndexedComparator.compareIndices(DataTransferer.CharsetComparator.charsets, encoding, encoding2, DataTransferer.CharsetComparator.OTHER_CHARSET_INDEX);
            if (compareIndices == 0) {
                return encoding2.compareTo(encoding);
            }
            return compareIndices;
        }
        
        protected static String getEncoding(String s) {
            if (s == null) {
                return null;
            }
            if (!DataTransferer.isEncodingSupported(s)) {
                return "UNSUPPORTED";
            }
            String canonicalName = DataTransferer.canonicalName(s);
            return DataTransferer.CharsetComparator.charsets.containsKey(canonicalName) ? canonicalName : s;
        }
        
        static {
            DEFAULT_CHARSET_INDEX = Integer.valueOf(2);
            OTHER_CHARSET_INDEX = Integer.valueOf(1);
            WORST_CHARSET_INDEX = Integer.valueOf(0);
            UNSUPPORTED_CHARSET_INDEX = Integer.valueOf(Integer.MIN_VALUE);
            HashMap m = new HashMap(8, 1.0f);
            m.put(DataTransferer.canonicalName("UTF-16LE"), Integer.valueOf(4));
            m.put(DataTransferer.canonicalName("UTF-16BE"), Integer.valueOf(5));
            m.put(DataTransferer.canonicalName("UTF-8"), Integer.valueOf(6));
            m.put(DataTransferer.canonicalName("UTF-16"), Integer.valueOf(7));
            m.put(DataTransferer.canonicalName("US-ASCII"), DataTransferer.CharsetComparator.WORST_CHARSET_INDEX);
            DataTransferer.canonicalName(DataTransferer.getDefaultTextCharset());
            if (m.get(DataTransferer.CharsetComparator.defaultEncoding) == null) {
                m.put(DataTransferer.CharsetComparator.defaultEncoding, DataTransferer.CharsetComparator.DEFAULT_CHARSET_INDEX);
            }
            m.put("UNSUPPORTED", DataTransferer.CharsetComparator.UNSUPPORTED_CHARSET_INDEX);
            charsets = Collections.unmodifiableMap(m);
        }
    }
    
    public static class DataFlavorComparator extends DataTransferer.IndexedComparator
    {
        private final DataTransferer.CharsetComparator charsetComparator;
        private static final Map exactTypes;
        private static final Map primaryTypes;
        private static final Map nonTextRepresentations;
        private static final Map textTypes;
        private static final Map decodedTextRepresentations;
        private static final Map encodedTextRepresentations;
        private static final Integer UNKNOWN_OBJECT_LOSES;
        private static final Integer UNKNOWN_OBJECT_WINS;
        private static final Long UNKNOWN_OBJECT_LOSES_L;
        private static final Long UNKNOWN_OBJECT_WINS_L;
        
        public DataFlavorComparator() {
            this(true);
        }
        
        public DataFlavorComparator(boolean b) {
            super(b);
            this.charsetComparator = new CharsetComparator(b);
        }
        
        @Override
        public int compare(Object o, Object o2) {
            DataFlavor dataFlavor;
            DataFlavor that;
            if (this.order) {
                dataFlavor = (DataFlavor)o;
                that = (DataFlavor)o2;
            }
            else {
                dataFlavor = (DataFlavor)o2;
                that = (DataFlavor)o;
            }
            if (dataFlavor.equals(that)) {
                return 0;
            }
            String primaryType = dataFlavor.getPrimaryType();
            String string = primaryType + "/" + dataFlavor.getSubType();
            Class<Object> representationClass = dataFlavor.getRepresentationClass();
            String primaryType2 = that.getPrimaryType();
            String string2 = primaryType2 + "/" + that.getSubType();
            Class<Object> representationClass2 = that.getRepresentationClass();
            if (dataFlavor.isFlavorTextType() && that.isFlavorTextType()) {
                int compareIndices4 = DataTransferer.IndexedComparator.compareIndices(DataTransferer.DataFlavorComparator.textTypes, string, string2, DataTransferer.DataFlavorComparator.UNKNOWN_OBJECT_LOSES);
                if (compareIndices4 != 0) {
                    return compareIndices4;
                }
                if (DataTransferer.doesSubtypeSupportCharset(dataFlavor)) {
                    int compareIndices5 = DataTransferer.IndexedComparator.compareIndices(DataTransferer.DataFlavorComparator.decodedTextRepresentations, representationClass, representationClass2, DataTransferer.DataFlavorComparator.UNKNOWN_OBJECT_LOSES);
                    if (compareIndices5 != 0) {
                        return compareIndices5;
                    }
                    int compareCharsets = this.charsetComparator.compareCharsets(DataTransferer.getTextCharset(dataFlavor), DataTransferer.getTextCharset(that));
                    if (compareCharsets != 0) {
                        return compareCharsets;
                    }
                }
                int compareIndices6 = DataTransferer.IndexedComparator.compareIndices(DataTransferer.DataFlavorComparator.encodedTextRepresentations, representationClass, representationClass2, DataTransferer.DataFlavorComparator.UNKNOWN_OBJECT_LOSES);
                if (compareIndices6 != 0) {
                    return compareIndices6;
                }
            }
            else {
                if (dataFlavor.isFlavorTextType()) {
                    return 1;
                }
                if (that.isFlavorTextType()) {
                    return -1;
                }
                int compareIndices = DataTransferer.IndexedComparator.compareIndices(DataTransferer.DataFlavorComparator.primaryTypes, primaryType, primaryType2, DataTransferer.DataFlavorComparator.UNKNOWN_OBJECT_LOSES);
                if (compareIndices != 0) {
                    return compareIndices;
                }
                int compareIndices2 = DataTransferer.IndexedComparator.compareIndices(DataTransferer.DataFlavorComparator.exactTypes, string, string2, DataTransferer.DataFlavorComparator.UNKNOWN_OBJECT_WINS);
                if (compareIndices2 != 0) {
                    return compareIndices2;
                }
                int compareIndices3 = DataTransferer.IndexedComparator.compareIndices(DataTransferer.DataFlavorComparator.nonTextRepresentations, representationClass, representationClass2, DataTransferer.DataFlavorComparator.UNKNOWN_OBJECT_LOSES);
                if (compareIndices3 != 0) {
                    return compareIndices3;
                }
            }
            return dataFlavor.getMimeType().compareTo(that.getMimeType());
        }
        
        static {
            UNKNOWN_OBJECT_LOSES = Integer.valueOf(Integer.MIN_VALUE);
            UNKNOWN_OBJECT_WINS = Integer.valueOf(Integer.MAX_VALUE);
            UNKNOWN_OBJECT_LOSES_L = Long.valueOf(Long.MIN_VALUE);
            UNKNOWN_OBJECT_WINS_L = Long.valueOf(Long.MAX_VALUE);
            HashMap m = new HashMap(4, 1.0f);
            m.put("application/x-java-file-list", Integer.valueOf(0));
            m.put("application/x-java-serialized-object", Integer.valueOf(1));
            m.put("application/x-java-jvm-local-objectref", Integer.valueOf(2));
            m.put("application/x-java-remote-object", Integer.valueOf(3));
            exactTypes = Collections.unmodifiableMap(m);
            HashMap i = new HashMap(1, 1.0f);
            i.put("application", Integer.valueOf(0));
            primaryTypes = Collections.unmodifiableMap(i);
            HashMap j = new HashMap(3, 1.0f);
            j.put(InputStream.class, Integer.valueOf(0));
            j.put(Serializable.class, Integer.valueOf(1));
            Class<Object> remoteClass = DataTransferer.RMI.remoteClass();
            if (remoteClass != null) {
                j.put(remoteClass, Integer.valueOf(2));
            }
            nonTextRepresentations = Collections.unmodifiableMap(j);
            HashMap k = new HashMap(16, 1.0f);
            k.put("text/plain", Integer.valueOf(0));
            k.put("application/x-java-serialized-object", Integer.valueOf(1));
            k.put("text/calendar", Integer.valueOf(2));
            k.put("text/css", Integer.valueOf(3));
            k.put("text/directory", Integer.valueOf(4));
            k.put("text/parityfec", Integer.valueOf(5));
            k.put("text/rfc822-headers", Integer.valueOf(6));
            k.put("text/t140", Integer.valueOf(7));
            k.put("text/tab-separated-values", Integer.valueOf(8));
            k.put("text/uri-list", Integer.valueOf(9));
            k.put("text/richtext", Integer.valueOf(10));
            k.put("text/enriched", Integer.valueOf(11));
            k.put("text/rtf", Integer.valueOf(12));
            k.put("text/html", Integer.valueOf(13));
            k.put("text/xml", Integer.valueOf(14));
            k.put("text/sgml", Integer.valueOf(15));
            textTypes = Collections.unmodifiableMap(k);
            HashMap l = new HashMap(4, 1.0f);
            l.put(char[].class, Integer.valueOf(0));
            l.put(CharBuffer.class, Integer.valueOf(1));
            l.put(String.class, Integer.valueOf(2));
            l.put(Reader.class, Integer.valueOf(3));
            decodedTextRepresentations = Collections.unmodifiableMap(l);
            HashMap m2 = new HashMap(3, 1.0f);
            m2.put(byte[].class, Integer.valueOf(0));
            m2.put(ByteBuffer.class, Integer.valueOf(1));
            m2.put(InputStream.class, Integer.valueOf(2));
            encodedTextRepresentations = Collections.unmodifiableMap(m2);
        }
    }
    
    public static class IndexOrderComparator extends DataTransferer.IndexedComparator
    {
        private final Map indexMap;
        private static final Integer FALLBACK_INDEX;
        
        public IndexOrderComparator(Map indexMap) {
            super(true);
            this.indexMap = indexMap;
        }
        
        public IndexOrderComparator(Map indexMap, boolean b) {
            super(b);
            this.indexMap = indexMap;
        }
        
        @Override
        public int compare(Object o, Object o2) {
            if (!this.order) {
                return -DataTransferer.IndexedComparator.compareIndices(this.indexMap, o, o2, DataTransferer.IndexOrderComparator.FALLBACK_INDEX);
            }
            return DataTransferer.IndexedComparator.compareIndices(this.indexMap, o, o2, DataTransferer.IndexOrderComparator.FALLBACK_INDEX);
        }
        
        static {
            FALLBACK_INDEX = Integer.valueOf(Integer.MIN_VALUE);
        }
    }
    
    private static class RMI
    {
        private static final Class<?> remoteClass;
        private static final Class<?> marshallObjectClass;
        private static final Constructor<?> marshallCtor;
        private static final Method marshallGet;
        
        private static Class<?> getClass(String name) {
            try {
                return Class.forName(name, true, null);
            }
            catch (ClassNotFoundException ex) {
                return null;
            }
        }
        
        private static Constructor<?> getConstructor(Class<?> clazz, Class<?>... parameterTypes) {
            try {
                return (clazz == null) ? null : clazz.getDeclaredConstructor(parameterTypes);
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        private static Method getMethod(Class<?> clazz, String name, Class<?>... parameterTypes) {
            try {
                return (clazz == null) ? null : clazz.getMethod(name, parameterTypes);
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        static boolean isRemote(Class<?> clazz) {
            return (boolean)((DataTransferer.RMI.remoteClass == null) ? null : Boolean.valueOf(DataTransferer.RMI.remoteClass.isAssignableFrom(clazz)));
        }
        
        static Class<?> remoteClass() {
            return DataTransferer.RMI.remoteClass;
        }
        
        static Object newMarshalledObject(Object o) throws IOException {
            try {
                return DataTransferer.RMI.marshallCtor.newInstance(new Object[] { o });
            }
            catch (InstantiationException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (IllegalAccessException detailMessage2) {
                throw new AssertionError(detailMessage2);
            }
            catch (InvocationTargetException detailMessage3) {
                Throwable cause = detailMessage3.getCause();
                if ((cause instanceof IOException)) {
                    throw (IOException)cause;
                }
                throw new AssertionError(detailMessage3);
            }
        }
        
        static Object getMarshalledObject(Object obj) throws IOException, ClassNotFoundException {
            try {
                return DataTransferer.RMI.marshallGet.invoke(obj, new Object[0]);
            }
            catch (IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (InvocationTargetException detailMessage2) {
                Throwable cause = detailMessage2.getCause();
                if ((cause instanceof IOException)) {
                    throw (IOException)cause;
                }
                if ((cause instanceof ClassNotFoundException)) {
                    throw (ClassNotFoundException)cause;
                }
                throw new AssertionError(detailMessage2);
            }
        }
        
        static {
            remoteClass = getClass("java.rmi.Remote");
            marshallObjectClass = getClass("java.rmi.MarshalledObject");
            marshallCtor = getConstructor(DataTransferer.RMI.marshallObjectClass, new Class[] { Object.class });
            marshallGet = getMethod(DataTransferer.RMI.marshallObjectClass, "get", new Class[0]);
        }
    }
}
